@@ -40,6 +40,7 @@ Examples
 >>> # graph_data = np.random.rand(10, 10) # full rank matrix
 >>> # graph_data = networkx.karate_club_graph() # networkx Graph/DiGraph objects
 >>> # graph_data = igraph.Graph.Famous('Zachary') # igraph Graph objects
+>>> # graph_data = graph_tool.collection.data['karate'] # graph_tool Graph objects
 >>>
 >>> # Create a non-interactive plot:
 >>> Graph(graph_data)
@@ -54,7 +55,7 @@ Examples
 >>> plt.show()
 """
 
-__version__ = "4.4.1"
+__version__ = "4.9.0"
 __author__ = "Paul Brodersen"
 __email__ = "paulbrodersen+netgraph@gmail.com"
 
@@ -68,9 +69,12 @@ from ._node_layout import (
     get_random_layout,
     get_fruchterman_reingold_layout,
     get_sugiyama_layout,
+    get_radial_tree_layout,
     get_circular_layout,
     get_linear_layout,
     get_bipartite_layout,
+    get_multipartite_layout,
+    get_shell_layout,
     get_community_layout,
 )
 
@@ -100,6 +104,7 @@ __all__ = [
     'get_random_layout',
     'get_fruchterman_reingold_layout',
     'get_sugiyama_layout',
+    'get_radial_tree_layout',
     'get_circular_layout',
     'get_linear_layout',
     'get_bipartite_layout',
