@@ -26,6 +26,7 @@ from ._utils import (
     _get_subgraph,
     _invert_dict,
     _get_connected_components,
+    _convert_polar_to_cartesian_coordinates,
 )
 
 
@@ -571,7 +572,8 @@ def _rescale_to_frame(node_positions, origin, scale):
     """Rescale node positions such that all nodes are within the bounding box."""
     node_positions = node_positions.copy() # force copy, as otherwise the `fixed_nodes` argument is effectively ignored
     node_positions -= np.min(node_positions, axis=0)
-    node_positions /= np.max(node_positions, axis=0)
+    # normalize only when nodes are not already aligned, otherwise we divide by zero
+    np.divide(node_positions, np.max(node_positions, axis=0), where=np.max(node_positions, axis=0) != 0, out=node_positions)
     node_positions *= scale[None, ...]
     node_positions += origin[None, ...]
     return node_positions
@@ -675,6 +677,73 @@ class vertex_view(object):
         self.h = h
 
 
+@_handle_multiple_components
+def get_radial_tree_layout(edges, origin=(0,0), scale=(1,1), node_size=3, total_iterations=3):
+    """Radial tree layout.
+
+    Uses the Sugiyama algorithm [Sugiyama1981]_ to compute node positions.
+    This function is a wrapper around the SugiyamaLayout class in grandalf.
+
+    Parameters
+    ----------
+    edges : list
+        The edges of the graph, with each edge being represented by a (source node ID, target node ID) tuple.
+    origin : tuple or None, default None
+        The (float x, float y) coordinates corresponding to the lower left hand corner of the bounding box specifying the extent of the canvas.
+        If None is given, the origin is placed at (0, 0).
+    scale : tuple or None, default None
+        The (float x, float y) dimensions representing the width and height of the bounding box specifying the extent of the canvas.
+        If None is given, the scale is set to (1, 1).
+    total_iterations : int, default 50
+        Number of iterations.
+
+    Returns
+    -------
+    node_positions : dict
+        Dictionary mapping each node ID to (float x, float y) tuple, the node position.
+
+    References
+    ----------
+    .. [Sugiyama1981] Sugiyama, K; Tagawa, S; Toda, M (1981) 'Methods for visual understanding of hierarchical system structures',
+           IEEE Transactions on Systems, Man, and Cybernetics
+
+    """
+
+    sugiyama_positions = get_sugiyama_layout(
+        edges,
+        origin=(0,0),
+        scale=(1,1),
+        node_size=node_size,
+        total_iterations=total_iterations
+    )
+
+    # determine the size of the largest layer
+    nodes_per_layer = dict()
+    for node, (_, y) in sugiyama_positions.items():
+        if y in nodes_per_layer:
+            nodes_per_layer[y] += 1
+        else:
+            nodes_per_layer[y] = 1
+    max_nodes_per_layer = np.max(list(nodes_per_layer.values()))
+
+    # alternatively:
+    # y_coordinates = np.array(list(sugiyama_positions.values()))[:, 1]
+    # (_, max_nodes_per_layer), = Counter(y_coordinates).most_common(1)
+
+    max_angle = 2 * np.pi * max_nodes_per_layer / (max_nodes_per_layer + 1)
+    max_radius = np.min(scale) / 2
+    max_radius *= 0.9 # shrink to make room for node artists, labels, and annotations
+    offset = np.array([origin[0] + 0.5 * scale[0], origin[1] + 0.5 * scale[1]])
+
+    node_positions = dict()
+    for node, (x, y) in sugiyama_positions.items():
+        angle = max_angle * x
+        radius = max_radius * (1 - y)
+        node_positions[node] = _convert_polar_to_cartesian_coordinates(radius, angle) + offset
+
+    return node_positions
+
+
 @_handle_multiple_components
 def get_circular_layout(edges, origin=(0,0), scale=(1,1), node_order=None, reduce_edge_crossings=True):
     """Circular node layout.
@@ -1113,6 +1182,159 @@ def _reduce_crossings_bipartite(adjacency_list, left, right):
     return left, sorted(right_ranks, key=right_ranks.get)
 
 
+def get_multipartite_layout(edges, layers, layer_positions=None, origin=(0, 0), scale=(1, 1), reduce_edge_crossings=True, uniform_node_spacing=True):
+    """Layered node layout for a multipartite graph.
+
+    By default, this implementation uses a heuristic to arrange the nodes such that the edge crossings are reduced.
+
+    Parameters
+    ----------
+    edges : list
+        The edges of the graph, with each edge being represented by a (source node ID, target node ID) tuple.
+    layers : list
+        List of node subsets, one for each layer of the graph.
+    layer_positions : list, default None
+        A list of x-coordinates, one for each layer.
+        If None provided, layers are placed evenly between origin[0] and origin[0] + scale[0].
+    origin : tuple, default (0, 0)
+        The (float x, float y) coordinates corresponding to the lower left hand corner of the bounding box specifying the extent of the canvas.
+    scale : tuple, default (1, 1)
+        The (float x, float y) dimensions representing the width and height of the bounding box specifying the extent of the canvas.
+    reduce_edge_crossings : bool, default True
+        If True, attempts to reduce edge crossings via the algorithm outlined in [Eades1994]_.
+    uniform_node_spacing : bool, default True
+        If True, the spacing between nodes is uniform across layers.
+        Otherwise, nodes in each layer are distributed evenly across the full height of the canvas.
+
+    Returns
+    -------
+    node_positions : dict
+        Dictionary mapping each node ID to (float x, float y) tuple, the node position.
+
+    References
+    ----------
+    .. [Eades1994] Eades & Wormald (1994) Edge crossings in drawings of bipartite graphs.
+
+    """
+
+    # set the space between nodes
+    if uniform_node_spacing:
+        try:
+            node_spacings = scale[1] / (np.max([len(layer) for layer in layers]) - 1) * np.ones_like(layers, dtype=np.float)
+        except ZeroDivisionError:
+            # The graph has at most a single edge between each pair of layers.
+            node_spacings = np.ones_like(layers, dtype=np.float)
+    else:
+        node_spacings = np.ones_like(layers, dtype=np.float)
+        for ii, layer in enumerate(layers):
+            if len(layer) > 1:
+                node_spacings[ii] = 1./(len(layer) - 1)
+
+    # set the space between layers
+    if layer_positions is None:
+        layer_positions = np.linspace(origin[0], origin[0] + scale[0], len(layers))
+    else:
+        if (np.min(layer_positions) < origin[0]) or (np.max(layer_positions) > origin[0] + scale[0]):
+            import warnings
+            warnings.warn("Some layer positions are outside the bounding box defined by `origin` and `scale`.")
+
+    # fix positions of nodes in first layer
+    node_positions = dict()
+    node_positions.update(_get_node_positions_within_layer(layers[0], node_spacings[0], layer_positions[0], origin, scale))
+
+    # assign the position of nodes in subsequent layers
+    total_layers = len(layers)
+    for ii in range(1, total_layers):
+        left = layers[ii-1]
+        right = layers[ii]
+        union = set(left) | set(right)
+        edges_between_layers = [(source, target) for (source, target) in edges if (source in union) and (target in union)]
+        adjacency_list = _edge_list_to_adjacency_list(edges_between_layers, directed=False)
+
+        # add unconnected nodes
+        for node in left:
+            adjacency_list.setdefault(node, set())
+        for node in right:
+            adjacency_list.setdefault(node, set())
+
+        if reduce_edge_crossings:
+            if not _is_complete_bipartite(edges, left, right):
+                left, right = _reduce_crossings_bipartite(adjacency_list, left, right)
+
+        node_positions.update(_get_node_positions_within_layer(right, node_spacings[ii], layer_positions[ii], origin, scale))
+
+    return node_positions
+
+
+def _get_node_positions_within_layer(node_order, node_spacing, layer_position, origin, scale):
+    node_positions = dict()
+    xx = layer_position
+    y = node_spacing * np.arange(len(node_order))
+    y -= np.mean(y)
+    y += origin[1] + scale[1] / 2
+    for node, yy in zip(node_order, y):
+        node_positions[node] = np.array((xx, yy))
+    return node_positions
+
+
+def get_shell_layout(edges, shells, radii=None, origin=(0, 0), scale=(1, 1), reduce_edge_crossings=True):
+    """Shell layout.
+
+    This is a wrapper around `get_multipartite_layout` that arranges nodes in shells around a center instead of in layers.
+
+    Parameters
+    ----------
+    edges : list
+        The edges of the graph, with each edge being represented by a (source node ID, target node ID) tuple.
+    shells : list
+        List of node subsets, one for each shell of the graph.
+    radii : list, default None
+        List of radii, one for each shell.
+        If None, radii are chosen such that the shells are evenly spaced within the bounding box defined by origin and scale.
+    origin : tuple
+        The (float x, float y) coordinates corresponding to the lower left hand corner of the bounding box specifying the extent of the canvas.
+    scale : tuple
+        The (float x, float y) dimensions representing the width and height of the bounding box specifying the extent of the canvas.
+    reduce_edge_crossings : bool, default True
+        If True, attempts to reduce edge crossings via the algorithm outlined in [Eades1994]_.
+
+    Returns
+    -------
+    node_positions : dict
+        Dictionary mapping each node ID to (float x, float y) tuple, the node position.
+
+    """
+    if radii is None:
+        if len(shells[0]) == 1:
+            # Innermost shell consists of a single node, and hence should have no size.
+            radii = np.linspace(0, 0.9 * np.min(scale) / 2, len(shells))
+        else:
+            # Innermost shell consists of multiple nodes and should hence have a non-zero radius.
+            radii = np.linspace(0, 0.9 * np.min(scale) / 2, len(shells) + 1)[1:]
+
+    relative_radii = np.array(radii) / np.max(radii)
+    multipartite_positions = get_multipartite_layout(
+        edges, shells, layer_positions=relative_radii,
+        reduce_edge_crossings=reduce_edge_crossings,
+        uniform_node_spacing=False
+    )
+
+    node_to_shell = {node : shell for shell in shells for node in shell}
+
+    max_radius = np.max(radii)
+    center = np.array([origin[0] + 0.5 * scale[0], origin[1] + 0.5 * scale[1]])
+
+    node_positions = dict()
+    for node, (x, y) in multipartite_positions.items():
+        shell = node_to_shell[node]
+        max_angle = 2 * np.pi * len(shell) / (len(shell) + 1)
+        angle = max_angle * y
+        radius = max_radius * x
+        node_positions[node] = _convert_polar_to_cartesian_coordinates(radius, angle) + center
+
+    return node_positions
+
+
 @_handle_multiple_components
 def get_community_layout(edges, node_to_community, origin=(0,0), scale=(1,1)):
     """Community node layout for modular graphs.
@@ -1126,6 +1348,7 @@ def get_community_layout(edges, node_to_community, origin=(0,0), scale=(1,1)):
     2. Position the nodes within each community:
        For each community, create a new graph. Find a layout for the subgraph.
     3. Combine positions computed in in 1) and 3).
+    4. Rotate communities around their centroid to reduce the length of the edges between them.
 
     Parameters
     ----------
@@ -1158,14 +1381,9 @@ def get_community_layout(edges, node_to_community, origin=(0,0), scale=(1,1)):
 
     community_size = _get_community_sizes(node_to_community, scale)
     community_centroids = _get_community_positions(edges, node_to_community, community_size, origin, scale)
-    relative_node_positions = _get_node_positions(edges, node_to_community)
-
-    # combine positions
-    node_positions = dict()
-    for node, community in node_to_community.items():
-        xy = community_centroids[community]
-        delta = relative_node_positions[node] * community_size[community]
-        node_positions[node] = xy + delta
+    relative_node_positions = _get_within_community_positions(edges, node_to_community)
+    node_positions = _combine_positions(node_to_community, community_centroids, community_size, relative_node_positions)
+    node_positions = _rotate_communities(edges, node_to_community, community_centroids, node_positions)
 
     return node_positions
 
@@ -1213,16 +1431,74 @@ def _find_between_community_edges(edges, node_to_community):
     return between_community_edges
 
 
-def _get_node_positions(edges, node_to_community):
+def _get_within_community_positions(edges, node_to_community):
     """Positions nodes within communities."""
     community_to_nodes = _invert_dict(node_to_community)
     node_positions = dict()
-    for nodes in community_to_nodes.values():
+    for community, nodes in community_to_nodes.items():
         if len(nodes) > 1:
             subgraph = _get_subgraph(edges, list(nodes))
-            subgraph_node_positions = get_fruchterman_reingold_layout(
-                subgraph, origin=np.array([-1, -1]), scale=np.array([2, 2]))
-            node_positions.update(subgraph_node_positions)
+            if subgraph:
+                subgraph_node_positions = get_fruchterman_reingold_layout(
+                    subgraph, origin=np.array([-1, -1]), scale=np.array([2, 2]))
+                node_positions.update(subgraph_node_positions)
+            else:
+                warnings.warn(f"There are no connections within community {community}. The placement of of nodes within this community is arbitrary.")
+                node_positions.update({node : np.random.rand(2) * 2 + np.array([-1, -1]) for node in nodes})
         elif len(nodes) == 1:
             node_positions.update({nodes.pop() : np.array([0., 0.])})
     return node_positions
+
+
+def _combine_positions(node_to_community, community_centroids, community_size, relative_node_positions):
+    node_positions = dict()
+    for node, community in node_to_community.items():
+        xy = community_centroids[community]
+        delta = relative_node_positions[node] * community_size[community]
+        node_positions[node] = xy + delta
+    return node_positions
+
+
+def _rotate_communities(edges, node_to_community, community_centroids, node_positions, step_size=-0.01, max_iterations=100):
+
+    between_community_edges = [(source, target) for (source, target) in edges \
+                               if node_to_community[source] != node_to_community[target]]
+
+    for _ in range(max_iterations):
+
+        # compute torques
+        community_torque = {community : 0 for community in set(list(node_to_community.values()))}
+        for (source, target) in between_community_edges:
+            # source
+            community = node_to_community[source]
+            r = community_centroids[community] - node_positions[source]
+            delta = node_positions[target] - node_positions[source]
+            F = delta * np.linalg.norm(delta) # direction * distance**2
+            community_torque[community] += np.cross(r, F)
+
+            # target
+            community = node_to_community[target]
+            r = community_centroids[community] - node_positions[target]
+            delta = node_positions[source] - node_positions[target]
+            F = delta * np.linalg.norm(delta)
+            community_torque[community] += np.cross(r, F)
+
+        # TODO: compare new torque values to previous; abort if change is small
+
+        # update node positions
+        step_size = -0.1
+        for node, community in node_to_community.items():
+            node_positions[node] = _rotate(step_size * community_torque[community],
+                                           node_positions[node],
+                                           community_centroids[community])
+
+    return node_positions
+
+
+def _rotate(angle, points, origin=(0, 0)):
+    # https://stackoverflow.com/a/58781388/2912349
+    R = np.array([[np.cos(angle), -np.sin(angle)],
+                  [np.sin(angle),  np.cos(angle)]])
+    origin = np.atleast_2d(origin)
+    points = np.atleast_2d(points)
+    return np.squeeze((R @ (points.T-origin.T) + origin.T).T)
