@@ -17,14 +17,20 @@ def _handle_multigraphs(parser):
     def wrapped_parser(graph, *args, **kwargs):
         nodes, edges, edge_weight = parser(graph, *args, **kwargs)
 
-        if len(set(edges)) < len(edges):
-            msg = "Multi-graphs are not properly supported. Removing duplicate edges before plotting."
+        new_edges = list(set([(edge[0], edge[1]) for edge in edges]))
+        if len(new_edges) < len(edges):
+            msg = "Multi-graphs are not properly supported. Duplicate edges are plotted as a single edge; edge weights (if any) are summed."
             warnings.warn(msg)
-            new_edges = []
-            for edge in edges:
-                if edge not in new_edges:
-                    new_edges.append(edge)
-            return nodes, new_edges, edge_weight
+            if edge_weight: # sum weights
+                new_edge_weight = dict()
+                for edge, weight in edge_weight.items():
+                    if (edge[0], edge[1]) in new_edge_weight:
+                        new_edge_weight[(edge[0], edge[1])] += weight
+                    else:
+                        new_edge_weight[(edge[0], edge[1])] = weight
+            else:
+                new_edge_weight = edge_weight
+            return nodes, new_edges, new_edge_weight
 
         return nodes, edges, edge_weight
 
@@ -47,8 +53,10 @@ def parse_graph(graph):
         - Adjacency matrix:
           Full-rank (V, V) ndarray (where V is the number of nodes/vertices).
           The absence of a connection is indicated by a zero.
-          Note that V > 3 as (2, 2) and (3, 3) matrices will be interpreted as edge lists.
-        - networkx.Graph or igraph.Graph object
+
+          .. note:: If V <= 3, any (2, 2) or (3, 3) matrices will be interpreted as edge lists.**
+
+        - networkx.Graph, igraph.Graph, or graph_tool.Graph object
 
     Returns
     -------
@@ -89,8 +97,11 @@ def parse_graph(graph):
                                   "<class 'networkx.classes.multidigraph.MultiDiGraph'>"):
         return _parse_networkx_graph(graph)
 
+    elif str(graph.__class__) == "<class 'graph_tool.Graph'>":
+        return _parse_graph_tool_graph(graph)
+
     else:
-        allowed = ['list', 'tuple', 'set', 'networkx.Graph', 'igraph.Graph']
+        allowed = ['list', 'tuple', 'set', 'networkx.Graph', 'igraph.Graph', 'graphtool.Graph']
         raise NotImplementedError("Input graph must be one of: {}\nCurrently, type(graph) = {}".format("\n\n\t" + "\n\t".join(allowed), type(graph)))
 
 
@@ -177,6 +188,16 @@ def _parse_igraph_graph(graph):
     return nodes, edges, edge_weights
 
 
+@_handle_multigraphs
+def _parse_graph_tool_graph(graph):
+    """Parse graphs given as graph_tool.Graph."""
+    nodes = graph.get_vertices().tolist()
+    edges = [tuple(edge) for edge in graph.get_edges()]
+    # In graph-tool, edge weights are in separate data structure called an edge property map.
+    edge_weights = None
+    return nodes, edges, edge_weights
+
+
 def _is_directed(edges):
     """Check if the edge list contains bi-directional edges, i.e. at least one edge (a, b) for which (b, a) also exists."""
     for (source, target) in edges:
